import logging
from sys import modules
from importlib import import_module, util

from aiohttp.web import run_app

from ..core import AbstractScript
from ..application import Application
from ..route import routes


class RunServer(AbstractScript):
    def main(self):
        settings = self.manager.settings
        logging.basicConfig(level=logging.DEBUG)

        run_app(
            self.app,
            loop=self.manager.loop,
            host=settings.host["ip"],
            port=settings.host["port"]
        )



import yaml
import os

__all__ = ["Settings"]


class Settings(object):
    """Class for parsing and representate settings
    """

    class YAMLParseException(Exception):
        pass

    def __init__(self):
        yaml.add_constructor(
            "tag:yaml.org,2002:map",
            self.dot_notated_dict.yaml_constructor
        )

    class dot_notated_dict(dict):
        def __getattr__(self, item):
            r = self.get(item)
            if r is not None:
                return r
            return super(Settings.dot_notated_dict, self).__getattribute__(item)

        @classmethod
        def yaml_constructor(cls, loader, node):
            values = loader.construct_mapping(node, deep=True)
            return Settings.dot_notated_dict(values)

    def update_from_yaml(self, yaml_filename: str):
        """
        :param yaml_filename
        :type yaml_filename: str
        """
        yaml_str = open(yaml_filename).read()
        yaml_dict = yaml.load(yaml_str)
        if not isinstance(yaml_dict, self.dot_notated_dict):
            raise Settings.YAMLParseException("YAML must be dict")
        self.__dict__.update(yaml_dict)
        if "$import" in yaml_dict:
            for import_file in yaml_dict["$import"]:
                try:
                    self.update_from_yaml(os.path.join(os.path.dirname(yaml_filename), import_file))
                except FileNotFoundError:
                    print("%s, imported from %s not found" % (import_file, yaml_filename))

    @staticmethod
    def from_yaml(yaml_filename: str):
        settings = Settings()
        settings.update_from_yaml(yaml_filename)
        return settings



import argparse
import asyncio
from sys import modules
from importlib import import_module, util
from puer.settings import Settings
from .scripts import scripts

from puer.application import Application
from puer.route import routes


__all__ = ['Manager']


class Manager(object):
    def __init__(self, args: argparse.Namespace, loop=None):
        self.args = args
        self.config_name = "%s.yaml" % self.args.config_name
        self.script_name = self.args.script_name
        self.settings = Settings.from_yaml(self.config_name)
        self.scripts = scripts
        self.loop = loop or asyncio.get_event_loop()
        self._load_apps(self.settings.apps)
        self._create_application()
        script = self.scripts[self.script_name](self)
        script.main()

    @staticmethod
    def parse_args():
        arg_parser = argparse.ArgumentParser(description='Manage script for Puer framework')
        arg_parser.add_argument('config_name', metavar='config_name', help='configuration file name '
                                                                 '(extension must be ".yaml")')
        arg_parser.add_argument('script_name', metavar='script_name', help='configuration name')
        return arg_parser.parse_args()

    def _load_apps(self, apps: list):
        for app in apps:
            self._load_scripts_from_app(app)
            self._load_urls_from_app(app)

    def _load_scripts_from_app(self, app):
        package_name = "%s.scripts" % app
        if util.find_spec(package_name) is not None:
            import_module(package_name)
            self.scripts.update(modules[package_name].scripts)

    def _load_urls_from_app(self, app):
        package_name = "%s.urls" % app
        if util.find_spec(package_name) is not None:
            import_module(package_name, app)

    def _create_application(self):
        # Setting middlewares for app
        middlewares = []
        for middleware in self.settings.middlewares:
            middleware_path = middleware.split('.')
            package_name = '.'.join(middleware_path[:-1])
            middleware_name = middleware_path[-1:][0]
            import_module(package_name)
            middlewares.append(getattr(modules[package_name], middleware_name))

        app = Application(
            middlewares=middlewares
        )
        app["settings"] = self.settings

        # Setting routes for app
        for res in routes:
            name = res[3]
            app.router.add_route(res[0], res[1], res[2], name=name)

        for m in self.settings.modules:
            module_splitted = m.split('.')
            package_name = '.'.join(module_splitted[0:-1])
            module_cls = module_splitted[-1]
            if util.find_spec(package_name) is not None:
                import_module(package_name)
                c = getattr(modules[package_name], module_cls)
                if c is not None:
                    mod = c(self, app)
                    app[mod.name] = mod.value

        # Setting signals for app
        for signal in self.settings.signals:
            app_signals = getattr(app, signal)
            for app_signal in self.settings.signals[signal]:
                app_signal_path = app_signal.split('.')
                package_name = '.'.join(app_signal_path[:-1])
                app_signal_name = app_signal_path[-1:][0]
                import_module(package_name)
                app_signals.append(getattr(modules[package_name], app_signal_name))

        self.application = app



from setuptools import setup, find_packages
from os.path import join, dirname
import puer

setup(
    name='puer',
    version=puer.__version__,
    packages=find_packages(exclude=('./venv',)),
    long_description=open(join(dirname(__file__), 'README.txt')).read(),
    entry_points={
        'console_scripts': [
            'puer_init = puer.puer_init:main'
        ]
    },
    install_requires=[
        'aiohttp',
        'pyyaml',
    ]
)


from abc import abstractmethod

__all__ = ["AbstractScript"]


class AbstractScript(object):
    def __init__(self, manager):
        self.manager = manager
        self.settings = manager.settings
        self.app = manager.application

    @abstractmethod
    def main(self):
        pass



import os


def create_folder(path, name):
    os.chdir(path)
    os.mkdir(name)


def main():
    cwd = os.getcwd()
    print("Current directory:", cwd)
    create_folder(cwd, "apps")
    print("Apps folder created")
    apps_dir = os.path.join(cwd, "apps")
    create_folder(apps_dir, "base")
    print("Base app folder created")
    with open(os.path.join(cwd, "manage.py"), "w") as manage_file:
        manage_file.write(
            """#!/usr/bin/env python3
from puer.manager import Manager


if __name__ == "__main__":
    args = Manager.parse_args()
    Manager(args)
"""
        )
        manage_file.close()
    os.chmod(os.path.join(cwd, "manage.py"), 775)
    print("Manage script created")
    with open(os.path.join(cwd, "base.yaml"), "w") as config_file:
        config_file.write(
            """apps: 
  - apps.base
middlewares: []
signals: []
host:
  ip: 127.0.0.1
  port: 8080
"""
        )
        config_file.close()
    print("Base config created")
